@@ -199,6 +199,8 @@ tp_dbus_check_valid_bus_name (const gchar *name,
   gchar last;
   const gchar *ptr;
 
+  g_return_val_if_fail (name != NULL, FALSE);
+
   if (name[0] == '\0')
     {
       g_set_error (error, TP_DBUS_ERRORS, TP_DBUS_ERROR_INVALID_BUS_NAME,
@@ -338,6 +340,8 @@ tp_dbus_check_valid_interface_name (const gchar *name,
   gchar last;
   const gchar *ptr;
 
+  g_return_val_if_fail (name != NULL, FALSE);
+
   if (name[0] == '\0')
     {
       g_set_error (error, TP_DBUS_ERRORS, TP_DBUS_ERROR_INVALID_INTERFACE_NAME,
@@ -442,6 +446,8 @@ tp_dbus_check_valid_member_name (const gchar *name,
 {
   const gchar *ptr;
 
+  g_return_val_if_fail (name != NULL, FALSE);
+
   if (name[0] == '\0')
     {
       g_set_error (error, TP_DBUS_ERRORS, TP_DBUS_ERROR_INVALID_MEMBER_NAME,
@@ -500,6 +506,8 @@ tp_dbus_check_valid_object_path (const gchar *path, GError **error)
 {
   const gchar *ptr;
 
+  g_return_val_if_fail (path != NULL, FALSE);
+
   if (path[0] != '/')
     {
       g_set_error (error, TP_DBUS_ERRORS, TP_DBUS_ERROR_INVALID_OBJECT_PATH,
@@ -710,7 +718,7 @@ _tp_dbus_daemon_got_name_owner (TpDBusDaemon *self,
 
 /**
  * TpDBusDaemonNameOwnerChangedCb:
- * @daemon: The D-Bus daemon
+ * @bus_daemon: The D-Bus daemon
  * @name: The name whose ownership has changed or been discovered
  * @new_owner: The unique name that now owns @name
  * @user_data: Arbitrary user-supplied data as passed to
@@ -749,6 +757,10 @@ tp_dbus_daemon_watch_name_owner (TpDBusDaemon *self,
   _NameOwnerWatch *watch = g_hash_table_lookup (self->priv->name_owner_watches,
       name);
 
+  g_return_if_fail (TP_IS_DBUS_DAEMON (self));
+  g_return_if_fail (name != NULL);
+  g_return_if_fail (callback != NULL);
+
   if (watch == NULL)
     {
       /* Allocate a single watch (common case) */
@@ -830,6 +842,10 @@ tp_dbus_daemon_cancel_name_owner_watch (TpDBusDaemon *self,
   _NameOwnerWatch *watch = g_hash_table_lookup (self->priv->name_owner_watches,
       name);
 
+  g_return_val_if_fail (TP_IS_DBUS_DAEMON (self), FALSE);
+  g_return_val_if_fail (name != NULL, FALSE);
+  g_return_val_if_fail (callback != NULL, FALSE);
+
   if (watch == NULL)
     {
       /* No watch at all */
@@ -988,14 +1004,19 @@ tp_dbus_daemon_class_init (TpDBusDaemonClass *klass)
  * (FIXME: should we also allow 'i' and 'u' with nonzero <=> True?)
  *
  * Returns: a boolean value for @key
- * @since 0.7.9
+ * Since: 0.7.9
  */
 gboolean
 tp_asv_get_boolean (const GHashTable *asv,
                     const gchar *key,
                     gboolean *valid)
 {
-  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+  GValue *value;
+
+  g_return_val_if_fail (asv != NULL, FALSE);
+  g_return_val_if_fail (key != NULL, FALSE);
+
+  value = g_hash_table_lookup ((GHashTable *) asv, key);
 
   if (value == NULL || !G_VALUE_HOLDS_BOOLEAN (value))
     {
@@ -1028,13 +1049,18 @@ tp_asv_get_boolean (const GHashTable *asv,
  * it for longer.
  *
  * Returns: the string value of @key, or %NULL
- * @since 0.7.9
+ * Since: 0.7.9
  */
 const GArray *
 tp_asv_get_bytes (const GHashTable *asv,
                    const gchar *key)
 {
-  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+  GValue *value;
+
+  g_return_val_if_fail (asv != NULL, NULL);
+  g_return_val_if_fail (key != NULL, NULL);
+
+  value = g_hash_table_lookup ((GHashTable *) asv, key);
 
   if (value == NULL || !G_VALUE_HOLDS (value, DBUS_TYPE_G_UCHAR_ARRAY))
     return NULL;
@@ -1057,13 +1083,18 @@ tp_asv_get_bytes (const GHashTable *asv,
  * need to keep it for longer.
  *
  * Returns: the string value of @key, or %NULL
- * @since 0.7.9
+ * Since: 0.7.9
  */
 const gchar *
 tp_asv_get_string (const GHashTable *asv,
                    const gchar *key)
 {
-  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+  GValue *value;
+
+  g_return_val_if_fail (asv != NULL, NULL);
+  g_return_val_if_fail (key != NULL, NULL);
+
+  value = g_hash_table_lookup ((GHashTable *) asv, key);
 
   if (value == NULL || !G_VALUE_HOLDS_STRING (value))
     return NULL;
@@ -1087,7 +1118,7 @@ tp_asv_get_string (const GHashTable *asv,
  * Otherwise, return 0, and if @valid is not %NULL, set *@valid to %FALSE.
  *
  * Returns: the 32-bit signed integer value of @key, or 0
- * @since 0.7.9
+ * Since: 0.7.9
  */
 gint32
 tp_asv_get_int32 (const GHashTable *asv,
@@ -1097,7 +1128,12 @@ tp_asv_get_int32 (const GHashTable *asv,
   gint64 i;
   guint64 u;
   gint32 ret;
-  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+  GValue *value;
+
+  g_return_val_if_fail (asv != NULL, 0);
+  g_return_val_if_fail (key != NULL, 0);
+
+  value = g_hash_table_lookup ((GHashTable *) asv, key);
 
   if (value == NULL)
     goto return_invalid;
@@ -1171,7 +1207,7 @@ return_invalid:
  * Otherwise, return 0, and if @valid is not %NULL, set *@valid to %FALSE.
  *
  * Returns: the 32-bit unsigned integer value of @key, or 0
- * @since 0.7.9
+ * Since: 0.7.9
  */
 guint32
 tp_asv_get_uint32 (const GHashTable *asv,
@@ -1181,7 +1217,12 @@ tp_asv_get_uint32 (const GHashTable *asv,
   gint64 i;
   guint64 u;
   guint32 ret;
-  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+  GValue *value;
+
+  g_return_val_if_fail (asv != NULL, 0);
+  g_return_val_if_fail (key != NULL, 0);
+
+  value = g_hash_table_lookup ((GHashTable *) asv, key);
 
   if (value == NULL)
     goto return_invalid;
@@ -1255,7 +1296,7 @@ return_invalid:
  * Otherwise, return 0, and if @valid is not %NULL, set *@valid to %FALSE.
  *
  * Returns: the 64-bit signed integer value of @key, or 0
- * @since 0.7.9
+ * Since: 0.7.9
  */
 gint64
 tp_asv_get_int64 (const GHashTable *asv,
@@ -1264,7 +1305,12 @@ tp_asv_get_int64 (const GHashTable *asv,
 {
   gint64 ret;
   guint64 u;
-  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+  GValue *value;
+
+  g_return_val_if_fail (asv != NULL, 0);
+  g_return_val_if_fail (key != NULL, 0);
+
+  value = g_hash_table_lookup ((GHashTable *) asv, key);
 
   if (value == NULL)
     goto return_invalid;
@@ -1327,7 +1373,7 @@ return_invalid:
  * Otherwise, return 0, and if @valid is not %NULL, set *@valid to %FALSE.
  *
  * Returns: the 64-bit unsigned integer value of @key, or 0
- * @since 0.7.9
+ * Since: 0.7.9
  */
 guint64
 tp_asv_get_uint64 (const GHashTable *asv,
@@ -1336,7 +1382,12 @@ tp_asv_get_uint64 (const GHashTable *asv,
 {
   gint64 tmp;
   guint64 ret;
-  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+  GValue *value;
+
+  g_return_val_if_fail (asv != NULL, 0);
+  g_return_val_if_fail (key != NULL, 0);
+
+  value = g_hash_table_lookup ((GHashTable *) asv, key);
 
   if (value == NULL)
     goto return_invalid;
@@ -1406,7 +1457,7 @@ return_invalid:
  * Otherwise, return 0.0, and if @valid is not %NULL, set *@valid to %FALSE.
  *
  * Returns: the double precision floating-point value of @key, or 0.0
- * @since 0.7.9
+ * Since: 0.7.9
  */
 gdouble
 tp_asv_get_double (const GHashTable *asv,
@@ -1414,7 +1465,12 @@ tp_asv_get_double (const GHashTable *asv,
                    gboolean *valid)
 {
   gdouble ret;
-  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+  GValue *value;
+
+  g_return_val_if_fail (asv != NULL, 0.0);
+  g_return_val_if_fail (key != NULL, 0.0);
+
+  value = g_hash_table_lookup ((GHashTable *) asv, key);
 
   if (value == NULL)
     goto return_invalid;
@@ -1476,13 +1532,18 @@ return_invalid:
  * need to keep it for longer.
  *
  * Returns: the object-path value of @key, or %NULL
- * @since 0.7.9
+ * Since: 0.7.9
  */
 const gchar *
 tp_asv_get_object_path (const GHashTable *asv,
                         const gchar *key)
 {
-  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+  GValue *value;
+
+  g_return_val_if_fail (asv != NULL, 0);
+  g_return_val_if_fail (key != NULL, 0);
+
+  value = g_hash_table_lookup ((GHashTable *) asv, key);
 
   if (value == NULL || !G_VALUE_HOLDS (value, DBUS_TYPE_G_OBJECT_PATH))
     return NULL;
@@ -1508,17 +1569,21 @@ tp_asv_get_object_path (const GHashTable *asv,
  * g_boxed_copy(), if you need to keep it for longer.
  *
  * Returns: the value of @key, or %NULL
- * @since 0.7.9
+ * Since: 0.7.9
  */
 gpointer
 tp_asv_get_boxed (const GHashTable *asv,
                   const gchar *key,
                   GType type)
 {
-  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+  GValue *value;
 
+  g_return_val_if_fail (asv != NULL, NULL);
+  g_return_val_if_fail (key != NULL, NULL);
   g_return_val_if_fail (G_TYPE_FUNDAMENTAL (type) == G_TYPE_BOXED, NULL);
 
+  value = g_hash_table_lookup ((GHashTable *) asv, key);
+
   if (value == NULL || !G_VALUE_HOLDS (value, type))
     return NULL;
 
@@ -1541,13 +1606,18 @@ tp_asv_get_boxed (const GHashTable *asv,
  * need to keep it for longer.
  *
  * Returns: the %NULL-terminated string-array value of @key, or %NULL
- * @since 0.7.9
+ * Since: 0.7.9
  */
 const gchar * const *
 tp_asv_get_strv (const GHashTable *asv,
                  const gchar *key)
 {
-  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+  GValue *value;
+
+  g_return_val_if_fail (asv != NULL, NULL);
+  g_return_val_if_fail (key != NULL, NULL);
+
+  value = g_hash_table_lookup ((GHashTable *) asv, key);
 
   if (value == NULL || !G_VALUE_HOLDS (value, G_TYPE_STRV))
     return NULL;
@@ -1568,11 +1638,14 @@ tp_asv_get_strv (const GHashTable *asv,
  * g_value_copy() if you need to keep it for longer.
  *
  * Returns: the value of @key, or %NULL
- * @since 0.7.9
+ * Since: 0.7.9
  */
 const GValue *
 tp_asv_lookup (const GHashTable *asv,
                const gchar *key)
 {
+  g_return_val_if_fail (asv != NULL, NULL);
+  g_return_val_if_fail (key != NULL, NULL);
+
   return g_hash_table_lookup ((GHashTable *) asv, key);
 }
