@@ -53,7 +53,7 @@ GType tp_dbus_daemon_get_type (void);
 
 TpDBusDaemon *tp_dbus_daemon_new (DBusGConnection *connection);
 
-typedef void (*TpDBusDaemonNameOwnerChangedCb) (TpDBusDaemon *daemon,
+typedef void (*TpDBusDaemonNameOwnerChangedCb) (TpDBusDaemon *bus_daemon,
     const gchar *name, const gchar *new_owner, gpointer user_data);
 
 void tp_dbus_daemon_watch_name_owner (TpDBusDaemon *self,
