@@ -1,7 +1,7 @@
 /*
  * util.c - Source for telepathy-glib utility functions
- * Copyright (C) 2006 Collabora Ltd.
- * Copyright (C) 2006 Nokia Corporation
+ * Copyright (C) 2006-2007 Collabora Ltd. <http://www.collabora.co.uk/>
+ * Copyright (C) 2006-2007 Nokia Corporation
  *   @author Robert McQueen <robert.mcqueen@collabora.co.uk>
  *
  * This library is free software; you can redistribute it and/or
@@ -158,6 +158,7 @@ tp_g_hash_table_update (GHashTable *target,
 
   g_return_if_fail (target != NULL);
   g_return_if_fail (source != NULL);
+  g_return_if_fail (target != source);
 
   g_hash_table_foreach (source, _tp_g_hash_table_update_helper, &data);
 }
@@ -297,3 +298,34 @@ tp_escape_as_identifier (const gchar *name)
     }
   return g_string_free (op, FALSE);
 }
+
+
+/**
+ * tp_strv_contains:
+ * @strv: a NULL-terminated array of strings, or %NULL (which is treated as an
+ *        empty strv)
+ * @str: a non-NULL string
+ *
+ * <!-- -->
+ * Returns: TRUE if @str is an element of @strv, according to strcmp().
+ *
+ * Since: 0.7.15
+ */
+gboolean
+tp_strv_contains (const gchar * const *strv,
+                  const gchar *str)
+{
+  g_return_val_if_fail (str != NULL, FALSE);
+
+  if (strv == NULL)
+    return FALSE;
+
+  while (*strv != NULL)
+    {
+      if (!tp_strdiff (str, *strv))
+        return TRUE;
+      strv++;
+    }
+
+  return FALSE;
+}
